"""Annotate group

Created by:
EOLE (http://eole.orion.education.fr)
Copyright (C) 2005-2018

Forked by:
Cadoles (http://www.cadoles.com)
Copyright (C) 2019-2021

distribued with GPL-2 or later license

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
"""
from rougail.i18n import _
from rougail.error import DictConsistencyError
from rougail.utils import normalize_family
from rougail.annotator.variable import Walk


class Annotator(Walk):
    """Annotate group
    """
    level = 10
    def __init__(self,
                 objectspace,
                 *args,
                 ):
        if not hasattr(objectspace.space, 'variables'):
            return
        self.objectspace = objectspace
        self.convert_groups()

    def convert_groups(self):  # pylint: disable=C0111
        """convert groups
        """
        # store old leaders family name
        for family in self.get_families():
            if not isinstance(family, self.objectspace.family):
                continue
            if not family.leadership:
                continue
            if hasattr(family, 'dynamic'):
                msg = _(f'the family "{family.name}" cannot be leadership and dynamic together')
                raise DictConsistencyError(msg, 31, family.xmlfiles)
            for idx, variable in enumerate(family.variable.values()):
                if idx == 0:
                    # it's a leader
                    if variable.multi is not True:
                        msg = _(f'the variable "{variable.name}" in a leadership must be multi')
                        raise DictConsistencyError(msg, 32, variable.xmlfiles)
                    family.hidden = variable.hidden
                    if variable.hidden:
                        variable.frozen = True
                        variable.force_default_on_freeze = True
                    variable.hidden = None
                else:
                    # it's a follower
                    if family.hidden:
                        variable.frozen = True
                        variable.force_default_on_freeze = True
                    if variable.multi is True:
                        variable.multi = 'submulti'
                    else:
                        variable.multi = True
